import React from 'react'
import { Query } from 'react-apollo'
import styled from 'styled-components'
import moment from 'moment'
import { Icon, Action } from '@pubsweet/ui'
import { th } from '@pubsweet/ui-toolkit'
import { B } from '../ui'
import { AllTypes } from '../upload-files'
import { GET_USER, GET_JOURNAL } from './operations'

const NoteEvent = styled.div`
  display: flex;
  align-items: flex-end;
  flex-wrap: wrap;
  justify-content: space-between;
  div {
    max-width: calc(100% - (${th('gridUnit')} * 4));
    font-size: ${th('fontSizeBaseSmall')};
  }
`
const EmailBody = styled.div`
  white-space: pre-wrap;
  padding-top: calc(${th('gridUnit')} * 2);
`
const Username = ({ id }) => (
  <Query query={GET_USER} variables={{ id }}>
    {({ data, loading }) => {
      if (loading || !data || !data.user) {
        return `...`
      }
      const { givenNames, surname } = data.user.identities[0].name
      return `${givenNames} ${surname}`
    }}
  </Query>
)

const Journal = ({ id }) => (
  <Query query={GET_JOURNAL} variables={{ id }}>
    {({ data, loading }) => {
      if (loading || !data || !data.journal) {
        return `...`
      }
      return <em>{data.journal.journalTitle}</em>
    }}
  </Query>
)

const toggleHidden = button => {
  const array = Array.from(button.children)
  array.forEach(element => {
    element.classList.toggle('hidden')
  })
  button.parentNode.nextElementSibling.classList.toggle('hidden')
}

const cleanUp = column => column.replace('meta,', '').replace('_', ' ')
const isJson = str => {
  try {
    JSON.parse(str)
  } catch (e) {
    return false
  }
  return true
}

const ParseJson = ({ col, val }) => {
  const set = isJson(val) && JSON.parse(val)
  switch (col) {
    case 'journal id':
      return val ? <Journal id={val} /> : '(unmatched)'
    case 'title':
      return <em>{val}</em>
    case 'article ids':
      return set.map(
        (p, i) =>
          `${p.pubIdType.toUpperCase()} ${p.id}${
            i !== set.length - 1 ? ', ' : ''
          }`,
      )
    case 'funding group':
      return set.map(
        (p, i) =>
          `${p.pi.surname}, ${p.fundingSource} ${p.awardId}${
            i !== set.length - 1 ? '; ' : ''
          }`,
      )
    case 'publication dates':
      return set.map((p, i) => {
        const type =
          (p.type === 'epub' && 'Electronic') || (p.type === 'ppub' && 'Print')
        const date = p.date
          ? moment(p.date).format('DD MMM YYYY')
          : `${p.jatsDate.year} ${p.jatsDate.season}`
        return `${date} (${type})${i !== set.length - 1 ? '; ' : ''}`
      })
    case 'location':
      if (set.fpage) {
        return `page ${set.fpage}${set.lpage && `-${set.lpage}`}`
      } else if (set.elocationId) {
        return `elocation ${set.elocationId}`
      }
      return ''
    default:
      return val
  }
}

const EventDescription = ({ audit, manuscript }) => {
  const { originalData, objectType, changes } = audit
  if (objectType === 'note') {
    const content = JSON.parse(changes.content)
    if (content && content.to) {
      return (
        <React.Fragment>
          <NoteEvent>
            <div>
              <B>Sent email to: </B>
              {content.to === 'helpdesk' ? (
                'Helpdesk'
              ) : (
                <Username id={content.to} />
              )}
              <br />
              <B>Subject: </B>
              {content.subject}
            </div>
            <Action
              onClick={e => toggleHidden(e.currentTarget)}
              title="Message body"
            >
              <Icon color="currentColor" size={3}>
                chevron-right
              </Icon>
              <Icon className="hidden" color="currentColor" size={3}>
                chevron-down
              </Icon>
            </Action>
          </NoteEvent>
          <EmailBody className="hidden">{content.message}</EmailBody>
        </React.Fragment>
      )
    }

    return `Note: ${content}`
  }

  if (objectType === 'team') {
    if (changes.role_name === 'submitter') {
      return `Became submitter`
    }
    if (changes.role_name === 'reviewer') {
      return `Became reviewer`
    }
  }

  if (objectType === 'file') {
    if (changes.type === 'manuscript') {
      return `Uploaded manuscript file: ${changes.filename}`
    }
    if (changes.filename) {
      return `Uploaded ${changes.filename}`
    }
    if (changes.type) {
      const type = AllTypes.find(t => t.value === changes.type).label
      return (
        <React.Fragment>
          Made <em>{originalData.filename}</em> type: {type}
        </React.Fragment>
      )
    }
    if (changes.label) {
      return (
        <React.Fragment>
          Gave <em>{originalData.filename}</em> label: {changes.label}
        </React.Fragment>
      )
    }
    if (changes.deleted) {
      return `Deleted ${originalData.filename}`
    }
  }

  if (objectType === 'manuscript') {
    if (changes.claimed_by) {
      return `Claimed manuscript`
    }
  }

  if (changes.deleted) {
    return `Deleted ${objectType}`
  }

  if (changes.id) {
    return `Created ${objectType}`
  }

  const keys = Object.keys(changes)
  const list = keys.filter(k => k !== 'updated_by')

  return (
    <React.Fragment>
      {list.map(k => (
        <React.Fragment key={k}>
          {`Set ${objectType} ${cleanUp(k)} to: `}
          <ParseJson col={cleanUp(k)} val={changes[k]} />
          <br />
        </React.Fragment>
      ))}
    </React.Fragment>
  )
}

export default EventDescription
