import React from 'react'
import { ApolloConsumer } from 'react-apollo'
import * as Joi from 'joi'
import { debounce } from 'lodash'
import { Button, ErrorText, H2, TextField } from '@pubsweet/ui'
import { SEARCH_JOURNALS } from './operations'
import {
  Buttons,
  HTMLString,
  Loading,
  LoadingIcon,
  SearchSelect,
  TextArea,
} from '../ui'

export default class UnmatchedCitation extends React.Component {
  state = {
    loading: false,
    title: this.props.title,
    query: this.props.journal ? this.props.journal.journalTitle : '',
    journal: this.props.journal,
    note: this.props.note,
    enabled: this.props.title && this.props.journal,
    journals: [],
    titleMessage: '',
    journalMessage: '',
  }
  enable = () => {
    const validTitle = Joi.validate(this.state.title, Joi.string().required())
    const validJournal = Joi.validate(
      this.state.journal,
      Joi.object({
        id: Joi.string(),
        journalTitle: Joi.string().required(),
        meta: Joi.object({
          pmcStatus: Joi.boolean(),
        }),
      }),
      { allowUnknown: true },
    )
    if (!validTitle.error && !validJournal.error) {
      this.setState({ enabled: true })
    } else {
      this.setState({
        titleMessage: validTitle.error ? 'Title is required.' : '',
        journalMessage: validJournal.error ? 'Journal is required.' : '',
      })
    }
  }
  submitData = () => {
    if (this.state.enabled) {
      const citationData = {
        meta: {
          title: this.state.title,
        },
      }
      if (this.state.journal.id) {
        citationData.meta.unmatchedJournal = null
        citationData.journalId = this.state.journal.id
      } else {
        citationData.meta.unmatchedJournal = this.state.journal.journalTitle
        citationData.journalId = null
      }
      if (this.state.note) {
        citationData.note = {
          notesType: 'userCitation',
          content: this.state.note,
        }
      }
      this.props.citation(citationData)
    }
  }
  onTitleChange = event => {
    this.setState(
      {
        titleMessage: '',
        title: event.target.value,
      },
      () => {
        this.enable()
      },
    )
  }
  onJournalSelect = journal => {
    const { id, journalTitle, meta } = journal
    this.setState(
      {
        journalMessage: '',
        journal: { id, journalTitle, meta },
        query: journal.journalTitle,
      },
      () => {
        this.enable()
      },
    )
  }
  render() {
    return (
      <div>
        <H2>Enter citation information</H2>
        <TextField
          invalidTest={this.state.titleMessage}
          label="Enter manuscript title"
          name="Title"
          onChange={this.onTitleChange}
          placeholder="The title of your manuscript"
          value={this.state.title}
        />
        {this.state.titleMessage && (
          <ErrorText>{this.state.titleMessage}</ErrorText>
        )}
        <ApolloConsumer>
          {client => {
            const searchJournals = debounce(async () => {
              const { query } = this.state
              const { data } = await client.query({
                query: SEARCH_JOURNALS,
                variables: { query },
              })
              this.setState({
                journals: data.alphaJournals,
                loading: false,
              })
            }, 500)
            const onQueryChange = e => {
              const query = e.target.value
              this.setState({
                query: e ? query : '',
                journals: [],
                loading: true,
              })
              if (query.trim().length > 0) {
                searchJournals()
              }
            }
            return (
              <SearchSelect
                invalidTest={this.state.journalMessage}
                label="Search for journal title"
                onInput={onQueryChange}
                optionsOnChange={this.onJournalSelect}
                placeholder="The journal title"
                query={this.state.query}
                singleSelect
              >
                {this.state.loading && (
                  <Loading>
                    <LoadingIcon />
                  </Loading>
                )}
                {this.state.journals.map(journal => (
                  <SearchSelect.Option
                    data-option={journal}
                    key={journal.id}
                    propKey={journal.id}
                  >
                    <HTMLString string={journal.journalTitle} />
                  </SearchSelect.Option>
                ))}
                {this.state.query.trim().length > 0 && (
                  <SearchSelect.Option
                    data-option={{ journalTitle: this.state.query }}
                    key="submit"
                    propKey="submit"
                  >
                    <em>
                      <HTMLString
                        string={`Submit &apos;${
                          this.state.query
                        }&apos; as your journal name`}
                      />
                    </em>
                  </SearchSelect.Option>
                )}
              </SearchSelect>
            )
          }}
        </ApolloConsumer>
        {this.state.journalMessage && (
          <ErrorText>{this.state.journalMessage}</ErrorText>
        )}
        <TextArea
          label="Other journal information (optional)"
          onChange={e => this.setState({ note: e.target.value })}
          placeholder="List URL, DOI, Volume/Issue, etc."
          rows={3}
          value={this.state.note}
        />
        <Buttons right>
          <Button
            disabled={!this.state.enabled}
            onClick={() => this.submitData()}
            primary
          >
            Submit
          </Button>
          {this.props.close && (
            <Button onClick={this.props.close}>Exit to citation search</Button>
          )}
        </Buttons>
      </div>
    )
  }
}
