import React from 'react'
import { withRouter } from 'react-router'
import { Mutation, Query } from 'react-apollo'
import moment from 'moment'
import styled from 'styled-components'
import { th, lighten } from '@pubsweet/ui-toolkit'
import { Button, H3, H4 } from '@pubsweet/ui'
import {
  Loading,
  LoadingIcon,
  Buttons,
  SectionContent as Content,
  SectionHeader as Header,
} from '../ui'
import { FileThumbnails } from '../preview-files'
import ReviewHistory from './ReviewHistory'
import { REVIEW_MANUSCRIPT, GET_REVIEWS } from './operations'

const Annotations = styled.ul`
  font-size: ${th('fontSizeBaseSmall')};
  list-style-type: none;
  padding-left: 0;
  margin-left: 0;
  li {
    padding: calc(${th('gridUnit')} * 2);
    border: ${th('borderWidth')} ${th('borderStyle')} ${th('colorBorder')};
    border-radius: calc(${th('borderRadius')} * 3);
    background-color: ${lighten('colorWarning', 85)};
    & + li {
      margin-top: calc(${th('gridUnit')} * 2);
    }
    p {
      font-size: ${th('fontSizeBaseSmall')};
      margin-bottom: 0;
    }
    &:after {
      content: '';
      display: block;
      clear: both;
    }
  }
`
const TimeStamp = styled.div`
  float: right;
  margin: 0 0;
  font-size: calc(${th('gridUnit')} * 1.5);
`
const Option = styled.div`
  display: flex;
  flex-flow: row wrap;
  align-items: center;
  justify-content: space-between;
  min-height: calc(${th('gridUnit')} * 6);
  padding: ${th('gridUnit')} calc(${th('gridUnit')} * 3);
  & > * {
    margin: ${th('gridUnit')} auto;
  }
  @media screen and (max-width: 870px) {
    padding: ${th('gridUnit')} calc(${th('gridUnit')} * 4);
  }
`
const ListErrors = ({ annotations }) => {
  const list = { tempHTML: [], pdf4print: [] }
  annotations.forEach(a => list[a.file.type].push(a))
  return (
    <React.Fragment>
      {list.tempHTML.length > 0 && (
        <React.Fragment>
          <H4>Web preview errors</H4>
          <AList list={list.tempHTML} />
        </React.Fragment>
      )}
      {list.pdf4print.length > 0 && (
        <React.Fragment>
          <H4>PDF preview errors</H4>
          <AList list={list.pdf4print} />
        </React.Fragment>
      )}
    </React.Fragment>
  )
}

const AList = ({ list }) => (
  <Annotations>
    {list.map(a => (
      <li key={a.id}>
        <em>{a.quote}</em>
        <p>{a.text}</p>
        <TimeStamp>{moment(a.updated).format('D/M/YY H:m')}</TimeStamp>
      </li>
    ))}
  </Annotations>
)

const ReviewFormDisplay = ({
  manuscript,
  previews,
  review,
  deleted,
  files,
  history,
}) => (
  <Mutation
    mutation={REVIEW_MANUSCRIPT}
    refetchQueries={() => [
      {
        query: GET_REVIEWS,
        variables: { manuscriptId: manuscript.id },
      },
    ]}
  >
    {(updateManuscript, { data }) => {
      const { annotations } = review || []
      const { id, status, teams } = manuscript
      const reviewer = teams.find(t => t.role === 'reviewer').teamMembers[0]
      const setStatus = async newStatus => {
        await updateManuscript({
          variables: { data: { id, status: newStatus } },
        })
      }
      if (status === 'tagging') {
        return (
          <React.Fragment>
            <Option>
              <H3>Tagging complete?</H3>
              <Button
                disabled={!previews}
                onClick={() => setStatus('xml-qa')}
                primary
              >
                Send for XML QA
              </Button>
            </Option>
            <Content>
              <p>
                {`When all files have been uploaded and both the Web and PDF previews have been generated, please click the button above to move the manuscript to XML QA.`}
              </p>
            </Content>
          </React.Fragment>
        )
      } else if (manuscript.status === 'xml-triage') {
        return (
          <React.Fragment>
            <Header>
              <H3>Review correction request</H3>
            </Header>
            <Content>
              <div>
                {annotations && <ListErrors annotations={annotations} />}
                {deleted.length > 0 && <ReviewHistory reviews={deleted} />}
                <p>
                  {`You can fix any errors and approve for review, or send this list of issues to the taggers to fix. You may edit the annotation text from the preview panes to provide more detail if necessary.`}
                </p>
                <p>
                  {`If an error is caused by missing or incomplete files, click 'Return for user upload' and you will be given the opportunity to write a message explaining what is needed, and send the manuscript back to the submitter or reviewer to upload files.`}
                </p>
                <Option>
                  <H3 style={{ width: '100%', textAlign: 'center' }}>
                    Errors fixed? Approve for:
                  </H3>
                  <Button onClick={() => setStatus('xml-qa')} primary>
                    QA
                  </Button>
                  <Button
                    onClick={async () => {
                      await setStatus('xml-review')
                      history.push('/')
                    }}
                    primary
                  >
                    Final review
                  </Button>
                </Option>
                <Option>
                  <H3>Tagger error?</H3>
                  <Button disabled>Confirm for retagging</Button>
                </Option>
                <Option>
                  <H3>Missing files?</H3>
                  <Button disabled>Return for user upload</Button>
                </Option>
              </div>
            </Content>
          </React.Fragment>
        )
      } else if (status === 'xml-qa' || status === 'xml-review') {
        if (status === 'xml-review') {
          deleted = deleted.filter(r => r.user.id === reviewer.user.id)
        }
        return (
          <React.Fragment>
            <Header>
              <H3>Review web versions</H3>
            </Header>
            <Content>
              <div>
                {status === 'xml-review' && (
                  <p>
                    {`Please review the web versions of your manuscript. This is the final step in the manuscript submission process.`}
                  </p>
                )}
                <p>
                  {`Preview both the PDF and Web versions ${
                    status === 'xml-review' ? 'of your' : 'generated from the'
                  } manuscript. Click '`}
                  <span className="hide-mobile">
                    Compare with submitted file&apos;, above,
                  </span>
                  <span className="show-mobile">Submitted file&apos;</span>
                  {` to view the originally submitted manuscript file. Other files submitted with the manuscript can be checked below.`}
                </p>
              </div>
            </Content>
            {files && files.length > 0 && (
              <React.Fragment>
                <Header>
                  <H3>Other submitted files</H3>
                </Header>
                <Content>
                  <FileThumbnails files={files} />
                </Content>
              </React.Fragment>
            )}
            <Header>
              <H3>Review</H3>
            </Header>
            <Content>
              <div>
                <p>
                  {`Approve Web and PDF previews, or mark any errors and request corrections.`}
                </p>
                {annotations && <ListErrors annotations={annotations} />}
                {deleted.length > 0 && <ReviewHistory reviews={deleted} />}
                <Buttons>
                  <Button
                    disabled={annotations}
                    onClick={async () => {
                      await setStatus(
                        status === 'xml-review' ? 'xml-triage' : 'xml-review',
                      )
                      history.push('/')
                    }}
                    primary
                  >
                    Approve
                  </Button>
                  {annotations && (
                    <Button
                      onClick={async () => {
                        await setStatus('xml-triage')
                        history.push('/')
                      }}
                    >
                      Send error report
                    </Button>
                  )}
                </Buttons>
              </div>
            </Content>
          </React.Fragment>
        )
      }
      history.push('/')
      return null
    }}
  </Mutation>
)

const ReviewFormRouter = withRouter(ReviewFormDisplay)

const ReviewForm = ({ manuscript, ...props }) => (
  <Query query={GET_REVIEWS} variables={{ manuscriptId: manuscript.id }}>
    {({ data: { deletedReviews }, loading }) => {
      if (loading || !deletedReviews) {
        return (
          <Loading>
            <LoadingIcon />
          </Loading>
        )
      }
      return (
        <ReviewFormRouter
          deleted={deletedReviews}
          manuscript={manuscript}
          {...props}
        />
      )
    }}
  </Query>
)

export default ReviewForm
